

#ifndef CONNECTION_H__
#define CONNECTION_H__
#include <stdio.h>
#include <stdbool.h>
#include "config.h"

typedef struct {
    unsigned char send_buffer[BUF_SIZE];
    unsigned char recv_buffer[BUF_SIZE];
    unsigned char gps_outfile[BUF_SIZE];
    unsigned char log_outfile[BUF_SIZE];
    unsigned char event_outfile[BUF_SIZE];
    unsigned char command_response_outfile[BUF_SIZE];
    unsigned char current_status_file[BUF_SIZE];
    unsigned char command_infile[BUF_SIZE];
    unsigned char current_packet[BUF_SIZE];
    FILE * log_filehandle;
    FILE * gps_filehandle;
    FILE * event_filehandle;
    FILE * command_response_filehandle;
    bool current_packet_valid;
    size_t send_count ;
    size_t read_count;
    size_t since_last_status;
    bool can_log;
    char imei[17];
    int socket;
} connection;


connection new_connection(int socket);
void logvfprintf( connection * conn, const char * format, va_list arglist) ;
void gpsvfprintf( connection * conn, const char * format, va_list arglist ) ;
void eventvfprintf( connection * conn, const char * format, va_list arglist ) ;
void logprintf( connection * conn, const char * format, ... );
void gpsprintf( connection * conn, const char * format, ... );
void eventprintf( connection * conn, const char * format, ... ) ;
void statusvfprintf(connection * conn, const char * format, va_list args) ;
void statusprintf(connection * conn, const char * format, ...) ;
void log_command_response(connection * conn, const unsigned char * response) ;
void log_time(connection * conn);
void log_line( connection * conn, const char * format, ... );
void log_array(connection * conn, uint8_t * array, size_t len);
void log_buffer(connection * conn);
void close_connection(connection * conn);
void init_imei(connection * conn);
#endif // CONNECTION_H_INCLUDED
