#include <memory.h>
#include "AM01_packet.h"
#include "../crc16.h"
#include "../connection.h"

bool is_v2(data_packet packet) {
    return packet.header.start_bit[0] == 0x79;
}

size_t data_length(data_packet packet) {
    return is_v2(packet)  ? min(BUF_SIZE - 20, (packet.header.length << 8) + packet.header.protocol_number - 5) : max(0, packet.header.length - 5);
}


// calculate 16 bits CRC of the given length data.
uint16_t crc16(data_packet packet) {
    uint16_t crc;
    uint16_t length = data_length(packet);

    if (!is_v2(packet)) {
        crc = crc16_addbyte(crc16_init(), packet.header.length);
        crc =  crc16_addbyte(crc, packet.header.protocol_number);

    } else {
        //unknown how the crc for a v2 packet is calculated
    }

    for (int i = 0; i < min(250, length); i++) {
        crc =  crc16_addbyte(crc, packet.data[i]);
    }

    crc = crc16_adduint16(crc, packet.footer.serial_number);
    crc = crc16_finish(crc);
    return crc;
}

data_packet get_basic_packet() {
    data_packet ret;
    ret.header.start_bit[0] = 0x78;
    ret.header.start_bit[1] = 0x78;
    ret.footer.stop_bit[0] = 0x0D;
    ret.footer.stop_bit[1] = 0x0A;
    ret.header.length = 5;
    return ret;
}

data_packet create_response(unsigned char protocol_number, unsigned  short count) {
    data_packet ret = get_basic_packet();
    ret.header.protocol_number = protocol_number;
    ret.footer.serial_number = count;
    ret.footer.crc = crc16(ret);
    return ret;
}

void send_data_packet( connection * conn, data_packet packet) {
    unsigned short length = packet.header.length - 5;
    int start = conn->send_count;
    memcpy(conn->send_buffer + start, &packet.header, sizeof(data_packet_header));
    conn->send_count += sizeof(data_packet_header);
    memcpy(conn->send_buffer + conn->send_count, packet.data, length);
    conn->send_count += length;
    memcpy(conn->send_buffer + conn->send_count, &packet.footer, sizeof(data_packet_footer));
    conn->send_count += sizeof(data_packet_footer);
    log_line(conn, "sent response: ");

    for (int i = start; i < conn->send_count; i++) {
        logprintf(conn, "%x ", ((unsigned char *)conn->send_buffer)[i]);
    }

    logprintf(conn, "\n");
}

void send_command(connection * conn, const char * cmd) {
    command_packet command;
    data_packet to_send = create_response(0x80, 0);
    memset(&command, 0, sizeof(command));
    memcpy(command.cmd, cmd, min(249, strlen(cmd)));
    command.length = 4 + strlen(command.cmd);
    memcpy(to_send.data, &command, command.length + 1);
    to_send.header.length = 6 + command.length;
    to_send.footer.crc = crc16(to_send);
    send_data_packet(conn, to_send);
}


void log_current_packet_ascii(connection * conn) {
    log_line(conn, "package data as ascii : ");

    for (int n = 0; n < data_length(PACKET(conn)); n++) {
        logprintf(conn, "%c ", PACKET(conn).data[n]);
    }

    logprintf(conn, "\n");
}

void log_current_packet(connection * conn) {
    log_line(conn, "current packet: ");
    log_array(conn, ((uint8_t *)&PACKET(conn).header), sizeof(data_packet_header));
    log_array(conn, PACKET(conn).data, data_length(PACKET(conn)));
    log_array(conn, ((uint8_t *)&PACKET(conn).footer), sizeof(data_packet_footer));
    logprintf(conn, "\n");
}
