#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <memory.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>

#include "connection.h"


//initial method to create a connection object
connection new_connection(int socket) {
    connection result;
    memset(&result, 0, sizeof(connection));
    result.can_log = false;
    result.imei[0] = 0;
    result.send_count = 0;
    result.read_count = 0;
    result.socket = socket;
    result.log_filehandle = 0;
    result.gps_filehandle = 0;
    result.since_last_status = time(0);
    return result;
}

bool openf(FILE ** fpp, unsigned char * fn) {
    if (*fpp <= 0) {
        *fpp = fopen(fn, "a");

        if (*fpp <= 0) {
            fprintf(stderr, "failed to open file for append: %s\n", fn);
            return false;
        }
    }

    return true;
}


void commandvfprintf( connection * conn, const char * format, va_list arglist ) {
    if (!conn->can_log || strlen(conn->command_response_outfile) < 16) {
        return;
    }

    if (!openf(&conn->command_response_filehandle, conn->command_response_outfile)) {
        return;
    }

    vfprintf( conn->command_response_filehandle, format, arglist );
    fflush(conn->command_response_filehandle);
    fsync(fileno(conn->command_response_filehandle));
}

void commandprintf( connection * conn, const char * format, ... ) {
    va_list arglist;
    va_start( arglist, format );
    commandvfprintf(conn, format, arglist);
    va_end( arglist );
}

void gpsvfprintf( connection * conn, const char * format, va_list arglist ) {
    if (!conn->can_log || strlen(conn->gps_outfile) < 16) {
        return;
    }

    if (!openf(&conn->gps_filehandle, conn->gps_outfile)) {
        return;
    }

    vfprintf( conn->gps_filehandle, format, arglist );
    fflush(conn->gps_filehandle);
    fsync(fileno(conn->gps_filehandle));
}

void gpsprintf( connection * conn, const char * format, ... ) {
    va_list arglist;
    va_start( arglist, format );
    gpsvfprintf(conn, format, arglist);
    va_end( arglist );
}

void logvfprintf( connection * conn, const char * format, va_list arglist) {
    if (!conn->can_log || strlen(conn->log_outfile) < 16) {
        return;
    }

    if (!openf(&conn->log_filehandle, conn->log_outfile)) {
        return;
    }

    vfprintf(conn->log_filehandle, format, arglist );
    fflush(conn->log_filehandle);
    fsync(fileno(conn->log_filehandle));
}

void logprintf( connection * conn, const char * format, ... ) {
    va_list arglist;
    va_start( arglist, format );
    logvfprintf(conn, format, arglist);
    va_end( arglist );
}

void eventvfprintf( connection * conn, const char * format, va_list arglist) {
    if (!conn->can_log || strlen(conn->event_outfile) < 16) {
        return;
    }

    if (!openf(&conn->event_filehandle, conn->event_outfile)) {
        return;
    }

    vfprintf(conn->event_filehandle, format, arglist );
    fflush(conn->event_filehandle);
    fsync(fileno(conn->event_filehandle));
}

void eventprintf( connection * conn, const char * format, ... ) {
    va_list arglist;
    va_start( arglist, format );
    eventvfprintf(conn, format, arglist);
    va_end( arglist );
}

void statusvfprintf(connection * conn, const char * format, va_list args) {
    FILE * fp;

    if (!conn->can_log || strlen(conn->event_outfile) < 16) {
        return;
    }

    fp = fopen (conn->current_status_file, "w");

    if (fp <= 0 ) {
        return;
    }

    vfprintf(fp, format, args);
    fclose (fp);
}

void statusprintf(connection * conn, const char * format, ...) {
    va_list arglist;
    va_start( arglist, format );
    statusvfprintf(conn, format, arglist);
    va_end( arglist );
}

void log_command_response(connection * conn, const unsigned char * response) {
    time_t t = time(NULL);
    struct tm tm = *gmtime(&t);
    commandprintf(conn, "%d-%02d-%02dT%02d:%02d:%02dZ,%s\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec, response);
}

void log_time(connection * conn) {
    time_t t = time(NULL);
    struct tm tm = *gmtime(&t);
    logprintf(conn, "%d-%02d-%02dT%02d:%02d:%02dZ,", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
}

void log_line( connection * conn, const char * format, ... ) {
    log_time(conn);
    va_list arglist;
    va_start( arglist, format );
    logvfprintf(conn, format, arglist);
    va_end( arglist );
}

void log_array(connection * conn, uint8_t * array, size_t len) {
    for (int n = 0; n < len; n++) {
        logprintf(conn, "%x ", array[n]);
    }
}

void log_buffer(connection * conn) {
    log_line(conn, "buffer contents : ");
    log_array(conn, conn->recv_buffer, conn->read_count);
    logprintf(conn, "\n");
}

void close_connection(connection * conn) {
    if (conn->log_filehandle) {
        fclose(conn->log_filehandle);
        conn->log_filehandle = 0;
    }

    if (conn->gps_filehandle) {
        fclose(conn->gps_filehandle);
        conn->gps_filehandle = 0;
    }

    if (conn->event_filehandle) {
        fclose(conn->event_filehandle);
        conn->event_filehandle = 0;
    }
}

void init_imei(connection * conn) {
    //convert our imei and set up paths
    memcpy(conn->gps_outfile, OUTDIR, strlen(OUTDIR) + 1);
    strcat(conn->gps_outfile, conn->imei);
    //base paths are all the same
    memcpy(conn->log_outfile, conn->gps_outfile, strlen(conn->gps_outfile) + 1);
    memcpy(conn->event_outfile, conn->gps_outfile, strlen(conn->gps_outfile) + 1);
    memcpy(conn->command_response_outfile, conn->gps_outfile, strlen(conn->gps_outfile) + 1);
    memcpy(conn->command_infile, conn->gps_outfile, strlen(conn->gps_outfile) + 1);
    memcpy(conn->current_status_file, conn->gps_outfile, strlen(conn->gps_outfile) + 1);
    //but extensions are not
    strcat(conn->gps_outfile, ".gps.txt");
    strcat(conn->log_outfile, ".log.txt");
    strcat(conn->event_outfile, ".event.txt");
    strcat(conn->command_response_outfile, ".command-output.txt");
    strcat(conn->command_infile, ".command.txt");
    strcat(conn->current_status_file, ".status.txt");
    //now that we've got a path/imei we can log things
    conn->can_log = true;
    log_line(conn, "imei recieved: %s \n", conn->imei);

    if (fileModifiedAgo(conn->gps_outfile) > 300) {
        DEFAULTS(conn);
    }
}
