/**
 * helper functions
 */
function tounix(date) {
    return parseInt((date.getTime() / 1000).toFixed(0)) + (date.getTimezoneOffset()*60);
}

function deg2rad(deg) {
    return deg * (Math.PI / 180)
}

function haversineDistance(lat1, lon1, lat2, lon2) {
    var R = 6371; // Radius of the earth in km
    var dLat = deg2rad(lat2 - lat1); // deg2rad below
    var dLon = deg2rad(lon2 - lon1);
    var a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
        Math.sin(dLon / 2) * Math.sin(dLon / 2);
    var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    var d = R * c; // Distance in km
    return d;
}


function distance(x, y, a, b) {
    return Math.sqrt((y - b) * (y - b) + (x - a) * (x - a));
}

function readableDate(dt) {
    return dt.toLocaleString();
}

function sumDistance(rows, beginIndex, endIndex) {
    var dist = 0.0;
    for (var i = beginIndex;
        (i < (rows.length - 1)) && (i < endIndex); i++) {
        dist += haversineDistance(rows[i][1], rows[i][2], rows[i + 1][1], rows[i + 1][2]);
    }
    return dist;
}

function distanceToMedian(rows, beginIndex, endIndex) {
    var dist = 0.0;
    var latSum = 0;
    var longSum = 0;
    var loopCount = 0;

    for (var i = beginIndex;
        (i < (rows.length - 1)) && (i < endIndex); i++) {
        dist += haversineDistance(rows[i][1], rows[i][2], rows[i + 1][1], rows[i + 1][2]);
    }
    return dist;
}

function combineDT(nameD, nameT) {
    const dtCompName='#'+nameD;
    const tCompName='#'+nameT;
    var dInput;
    var tInput;
    
    if ( $(dtCompName).prop('type') != 'date' ){
        dInput=$(dtCompName).val();
        tInput=$(tCompName).val();
    }else{
        dInput = document.getElementById(nameD).value;
        tInput= document.getElementById(nameT).value;
    }
        
    if (dInput && tInput) {
        return new Date(dInput + 'T' + tInput);
    }
    
    if (dInput) return new Date(dInput);

    return null;
}

function isRowVisible(el) {
    var rect = el.getBoundingClientRect();
    var top = rect.top;
    var height = rect.height;
    
    el = el.parentNode;

    // Check if bottom of the element is off the parent element
    if (rect.bottom < 0) return false;
    // Check its within the document viewport
    if (top > document.documentElement.clientHeight) return false;
    do {
        rect = el.getBoundingClientRect();
        if (top <= rect.bottom === false) return false;
        // Check if the element is out of view due to a container scrolling
        if ((top + height) <= rect.top) return false;
        el = el.parentNode;
    } while (el != document.body)
    return true;
};
