/**
 * Elements that make up the popup.
 */
var container = document.getElementById('popup');
var content = document.getElementById('popup-content');
var closer = document.getElementById('popup-closer');


/**
 * Add a click handler to hide the popup.
 * @return {boolean} Don't follow the href.
 */
closer.onclick = function() {
    overlay.setPosition(undefined);
    closer.blur();
    return false;
};

function travelLayerStyle(feature) {
    var styles = [
        // linestring
        new ol.style.Style({
            stroke: new ol.style.Stroke({
                color: 'blue',
                width: 4,
            }),
        })
    ];

    feature.getGeometry().forEachSegment(function(start, end) {
        var dx = end[0] - start[0];
        var dy = end[1] - start[1];
        var rotation = Math.atan2(dy, dx);
        // arrows
        styles.push(
            new ol.style.Style({
                geometry: new ol.geom.Point(end),
                image: new ol.style.Icon({
                    src: 'arrow.png',
                    anchor: [0.75, 0.5],
                    rotateWithView: true,
                    rotation: -rotation,
                }),
            })
        );
    });

    return styles;
}

const markerStyle = new ol.style.Style({
    image: new ol.style.Icon({
        anchor: [0.5, 0.5],
        size: [400, 600],
        offset: [0, 0],
        opacity: 1,
        scale: 0.12,
        src: "pin.png"
    })
});

var eventStyle = new ol.style.Style({
    image: new ol.style.Circle({
        radius: 10,
        fill: new ol.style.Fill({
            color: 'orange'
        })
    })
});

var overlay = new ol.Overlay({
    element: container,
    autoPan: true,
    autoPanAnimation: {
        duration: 250
    }
});

var map = new ol.Map({
    target: 'map',
    view: new ol.View({
        center: defaultCenter,
        zoom: 17
    }),
    overlays: [overlay],
    renderer: 'webgl'

});

var satLayer = new ol.layer.Tile({
    title: "Google Satellite",
    source: new ol.source.TileImage({
        url: 'http://mt1.google.com/vt/lyrs=s&hl=pl&&x={x}&y={y}&z={z}'
    }),
    visible: false
});

map.addLayer(satLayer);

var osmLayer = new ol.layer.Tile({
    title: "Open street maps",
    source: new ol.source.OSM(),
    minZoom: 19 // visible at zoom levels 14 and below
})

map.addLayer(osmLayer); 

var arcgisLayer = new ol.layer.Tile({
  title: "ArcGIS map",
  source: new ol.source.XYZ({
    url:'https://server.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer/tile/{z}/{y}/{x}',
  }),
  maxZoom: 19 // visible at zoom levels above 14
});

map.addLayer(arcgisLayer); //use arcGIS instead of open street map - it's much faster
/*
map.getView().on('change:resolution', function() {
    const layers = map.getLayers();
    const zoom=map.getView().getZoom();

    
    layers.forEach(function(layer, i, a) {
        if(layer && layer.getProperties() ){
            var visible=false;
            if( zoom > layer.getProperties().minZoom && zoom < layer.getProperties().maxZoom) visible=true;
            layer.setVisible(visible);
        }
    });
});
*/
var travelFeature = new ol.Feature({
    geometry: new ol.geom.LineString([])
});

var travelLayer = new ol.layer.VectorImage({
    title: "Route history",
    source: new ol.source.Vector({
        features: [
            travelFeature
        ]
    }),
    style: travelLayerStyle(travelFeature),
    renderMode: 'image'
});


map.addLayer(travelLayer);


var pointFeature = new ol.Feature(new ol.geom.Point(defaultCenter));
var currentPointLayer = new ol.layer.Vector({
                title: "Current location",
    source: new ol.source.Vector({
        features: [
            pointFeature
        ]
    }),
    style: markerStyle
});

map.addLayer(currentPointLayer);

var eventFeature = new ol.Feature(new ol.geom.MultiPoint([]));
var eventLayer = new ol.layer.Vector({
    source: new ol.source.Vector({
        features: [
            eventFeature
        ]
    }),
    eventStyle
});

map.addLayer(eventLayer);

map.getView().animate({
    center: ol.proj.fromLonLat([defaultCenter[0], defaultCenter[1]]),
    duration: 2000
})
