function changeLayer() {
    const layer = document.getElementById("layer").value;

    if (layer == "0") {
        satLayer.setVisible(false);
        arcgisLayer.setVisible(true);
        osmLayer.setVisible(true);
    } else {
        satLayer.setVisible(true);
        arcgisLayer.setVisible(false);
        osmLayer.setVisible(false);
    }
}

function updateCurrentPosition() {
    var url = "current.php?imei=" + imei;

    $.ajax({
        url: url,
        type: 'GET',
        cache: false,
        success: function(result) {
            var lines = result.split(/\r?\n/);
            var coords = lines[0].split(/,/);
            pointFeature.name = 'Last seen on: ' + readableDate(new Date(coords[0])) + ' <br>Moving at speed ' + coords[3] + 'km/h<br><br>' + lines[1].replace(/;/g, '<br>');
            pointFeature.getGeometry().setCoordinates(ol.proj.fromLonLat([coords[2], coords[1]]));
        }
    });
}

updateCurrentPosition();
setInterval(updateCurrentPosition, 10000);

map.on('singleclick', function(evt) {
    var name = '';

    map.forEachFeatureAtPixel(evt.pixel, function(feature) {
        if (feature.name) 
            name = pointFeature.name;
                              
        if (feature.extra) {
            //determine the closest event to the point that was clicked
            var coords = feature.getGeometry().getCoordinates();
            var closestPt = 0;
            var closestDistance = Number.MAX_VALUE;
            
            for (var i = 0; i < coords.length; i++) {
                var dist = distance(evt.coordinate[0], evt.coordinate[1], coords[i][0], coords[i][1]);
                if (dist < closestDistance) {
                    closestPt = i;
                    closestDistance = dist;
                }
            }
            
            name = feature.extra[closestPt];
        }
    })

    if (name != '') {
        container.style.display = "block";
        var coordinate = evt.coordinate;
        content.innerHTML = name;
        overlay.setPosition(coordinate);
    } else {
        container.style.display = "none";
    }
});


function animateTo(long, lat) {
    setfocus = 0;
    var lonlat = ol.proj.fromLonLat([long, lat]);
    map.getView().animate({
        center: lonlat,
        duration: 2000
    });
}

function constructURL(phpName, begindate, enddate) {
    return phpName + "?imei=" + imei + "&begin=" + tounix(begindate)+ (enddate? ( "&end=" + tounix(enddate)):"");
}

function forEachRow(page, minCols, func) {
    var lines = page.split(/\r?\n/);
    var ret = lines.map(line => line.split(',')).filter(cols => cols.length > minCols).map(cols => func(cols));
    return ret;
}

function computeEventRow(cols) {
    return "<tr onclick='animateTo(" + cols[2] + "," + cols[1] + ")'><td>" + readableDate(new Date(cols[0])) + "</td><td>" + cols[1] + "</td><td>" + cols[2] + "</td><td>" + cols[3] + "</td><td>" + cols[4] + "</td></tr>";
}

function computeHistoryRow(cols) {
    return "<tr onclick='animateTo(" + cols[2] + "," + cols[1] + ")'><td>" + readableDate(new Date(cols[0])) + "</td><td>" + cols[1] + "</td><td>" + cols[2] + "</td><td>" + cols[3] + "</td></tr>";
}

function computeLogRow(cols) {
    return "<tr><td>" + readableDate(new Date(cols[0])) + "</td><td>" + cols[1] + "</td></tr>";
}

function tableHeader(cols) {
    const tableHead = document.getElementById("dataHead");

    tableHead.innerHTML = '';

    cols.forEach(col => {
        th = document.createElement('th');
        th.innerHTML = col;
        tableHead.appendChild(th);
    });
}

function enableDownload(data, enabled) {
    const downloadbtn = document.getElementById("downloadbtn");

    if (enabled == false) {
        downloadbtn.enabled = false;
        return;
    }
    
    downloadbtn.enabled = true;
    downloadbtn.onclick = function() {
        var hiddenElement = document.createElement('a');
        hiddenElement.href = 'data:text/csv;charset=utf-8,' + encodeURI(data);
        hiddenElement.target = '_blank';

        //provide the name for the CSV file to be downloaded  
        hiddenElement.download = 'dataset.csv';
        hiddenElement.click();
    }
}
//for trip fetch events from start to end of trip
function fetchEvents(toTable, beginDate, endDate) {
    $.ajax({
        url: constructURL("events.php", beginDate, endDate),
        success: function(result) {
            enableDownload(result, true);
            var parsed = forEachRow(result, 4, cols => [new Date(cols[0]), parseFloat(cols[1]), parseFloat(cols[2]), parseFloat(cols[3]), cols[4]]);
            var coords = parsed.map(rv => (ol.proj.fromLonLat([rv[2], rv[1]])));
            var captions = parsed.map(rv => (rv[4] + ' on ' + readableDate(rv[0]) + ' while moving at speed: ' + rv[3]));

            eventFeature.getGeometry().setCoordinates(coords);
            eventFeature.extra = captions;
            eventLayer.setStyle(eventStyle);

            if (toTable) {
                tableHeader(['date', 'lattitude', 'longitude', 'speed', 'event']);
                const tableBody = document.getElementById("dataBody");
                tableBody.innerHTML = parsed.map(rv => computeEventRow(rv)).join('');
            }
        }
    });
}

function fetchHistory(beginDate, endDate) {
    $.ajax({
        url: constructURL("history.php", beginDate, endDate),
        success: function(result) {
            enableDownload(result, true);

            var parsed = forEachRow(result, 3, cols => [new Date(cols[0]), parseFloat(cols[1]), parseFloat(cols[2]), parseFloat(cols[3])]);
            var coords = parsed.map(rv => (ol.proj.fromLonLat([rv[2], rv[1]])));

            travelFeature.getGeometry().setCoordinates(coords);
            travelFeature.setStyle(travelLayerStyle(travelFeature));

            tableHeader(['date', 'lattitude', 'longitude', 'speed']);
            const tableBody = document.getElementById("dataBody");
            tableBody.innerHTML = parsed.map(rv => computeHistoryRow(rv)).join('');


            var totalDistance = 0;

            for (var i = 0; i < parsed.length - 1; i++) {
                totalDistance += haversineDistance(parsed[i][1], parsed[i][2], parsed[i + 1][1], parsed[i + 1][2])
            }
            var distanceDiv = document.getElementById("distance");

            distanceDiv.innerHTML = '  distance covered: ' + (Math.round(totalDistance * 100) / 100) + 'km';

        }
    });
}


function fetchLogging(beginDate, endDate) {
    $.ajax({
        url: constructURL("logging.php", beginDate, endDate),
        success: function(result) {
            enableDownload(result, true);

            var parsed = forEachRow(result, 1, cols => [new Date(cols[0]), cols.slice(1).join(',')]);

            travelFeature.getGeometry().setCoordinates([]);
            eventFeature.getGeometry().setCoordinates([]);

            tableHeader(['date', 'log line']);
            const tableBody = document.getElementById("dataBody");
            tableBody.innerHTML = parsed.map(rv => computeLogRow(rv)).join('');

        }
    });
}

function fetchCommandResults(beginDate, endDate) {
    $.ajax({
        url: constructURL("command_output.php", beginDate, endDate),
        success: function(result) {
            enableDownload(result, true);

            var parsed = forEachRow(result, 1, cols => [new Date(cols[0]), cols.slice(1).join(',')]);

            travelFeature.getGeometry().setCoordinates([]);
            eventFeature.getGeometry().setCoordinates([]);

            tableHeader(['date', 'command result']);
            const tableBody = document.getElementById("dataBody");
            tableBody.innerHTML = parsed.map(rv => computeLogRow(rv)).join('');

        }
    });
}


function fetchTrips(beginDate, endDate) {
    $.ajax({
        url: constructURL("history.php", beginDate, endDate),
        success: function(result) {
            enableDownload(result, false);

            var parsed = forEachRow(result, 3, cols => [new Date(cols[0]), parseFloat(cols[1]), parseFloat(cols[2]), parseFloat(cols[3])]);

            tableHeader(['start date', 'end date', 'start position', 'end position', 'distance traveled']);
            const tableBody = document.getElementById("dataBody");
            tableBody.innerHTML = '';
            convert_to_trips(tableBody, parsed);
        }
    });
}

function searchdateChange() {
    const events = document.getElementById("eventsel").value;
    const table = document.getElementById("coordinateTable");
    var firstRowTimestamp;

    for (i = 1; i < table.rows.length; i++) {
        let row = table.rows[i];
        if (isRowVisible(row)) {
            firstRowTimestamp = row.cells[0].innerHTML;
            break;
        }
    }

    const begindate = combineDT('beginDate', 'beginTime');
    const enddate = combineDT('endDate', 'endTime') ? combineDT('endDate', 'endTime') : null;


    switch (events) {
        case "0":
            fetchEvents(false, begindate, enddate);
            fetchHistory(begindate, enddate);
            break;
        case "1":
            fetchEvents(true, begindate, enddate);
            break;
        case "2":
            fetchLogging(begindate, enddate);
            break;
        case "3":
            fetchCommandResults(begindate, enddate);
            break;
        case "4":
            fetchTrips(begindate, enddate);
            break;
    }
    
    for (var i = 0; i < table.rows.length; i++) {
        let row = table.rows[i];
        if (row.cells[0].innerHTML == firstRowTimestamp) {
            row.scrollIntoView();
            break;
        }
    }
}

setInterval(searchdateChange, 80000);
searchdateChange();
